var UNDO = 'redux-undone/UNDO';
var REDO = 'redux-undone/REDO';
var undo = function undo() {
  return { type: UNDO };
};
var redo = function redo() {
  return { type: REDO };
};

function hasOne(obj, properties) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = properties[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var property = _step.value;

            if (has(obj, property)) {
                return true;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return false;
}
function has(obj, property) {
    var path = property.split('.');
    var current = obj;
    for (var i = 0; i < path.length; i++) {
        current = current[path[i]];
        if (i !== path.length - 1 && !current) {
            return false;
        }
    }
    return true;
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var isObject = function isObject(value) {
    return (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && !Array.isArray(value);
};
var merge = function merge(src, dst) {
    return Object.entries(dst).reduce(function (acc, _ref) {
        var _ref2 = slicedToArray(_ref, 2),
            dstProperty = _ref2[0],
            dstValue = _ref2[1];

        var srcValue = src[dstProperty];
        acc[dstProperty] = isObject(srcValue) && isObject(dstValue) ? merge(srcValue, dstValue) : dstValue;
        return acc;
    }, Object.assign({}, src));
};

var History = function () {
    function History() {
        classCallCheck(this, History);

        this.future = [];
        this.past = [];
    }

    createClass(History, [{
        key: "clearFuture",
        value: function clearFuture() {
            this.future = [];
        }
    }, {
        key: "getPast",
        value: function getPast() {
            return Object.freeze(this.past);
        }
    }, {
        key: "pop",
        value: function pop(past) {
            if (past) {
                return this.past.pop();
            }
            return this.future.pop();
        }
    }, {
        key: "push",
        value: function push(transformer) {
            var past = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            if (past) {
                return this.past.push(transformer);
            }
            return this.future.push(transformer);
        }
    }]);
    return History;
}();

// import logger from './logger';
var globallySkippedProperties = ['meta.reduxUndone.skip'];
function dispatch(store, action) {
    if (typeof action === 'function') {
        return action(dispatch.bind(null, store), store.getState);
    }
    var nonUndoableAction = merge(action, {
        meta: { reduxUndone: { skip: true } }
    });
    store.dispatch(nonUndoableAction);
    return nonUndoableAction;
}
function processAction(store, next, action) {
    if (typeof action === 'function') {
        return action(store.dispatch, store.getState);
    }
    return next(action);
}
function performDispatch(store, action) {
    var oldState = store.getState();
    var dispatchedAction = dispatch(store, action);
    var newState = store.getState();
    return { oldState: oldState, newState: newState, dispatchedAction: dispatchedAction };
}
function createTransformer(transformers, store, action, undoing) {
    var _performDispatch = performDispatch(store, action),
        oldState = _performDispatch.oldState,
        newState = _performDispatch.newState,
        dispatchedAction = _performDispatch.dispatchedAction;

    var transformer = transformers[dispatchedAction.type];
    if (!transformer) {
        // logger.log(`No transformation found for ${dispatchedAction.type}`, 'warn');
        return null;
    }
    return function () {
        return transformer(dispatchedAction, oldState, newState, undoing);
    };
}
function createMiddleware(transformers) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var history = new History();
    return function (store) {
        return function (next) {
            return function (action) {
                var type = action.type;
                if (type === UNDO || type === REDO) {
                    var undoing = type === UNDO;
                    var prevTransformer = history.pop(undoing);
                    if (prevTransformer == null) {
                        return;
                    }
                    var transformed = prevTransformer();
                    var _transformer = createTransformer(transformers, store, transformed, !undoing);
                    if (_transformer === null) {
                        return;
                    }
                    history.push(_transformer, !undoing);
                    return;
                }
                var skippedProperties = options.skippedProperties;

                if (hasOne(action, [].concat(globallySkippedProperties, toConsumableArray(skippedProperties || [])))) {
                    processAction(store, next, action);
                    return;
                }
                var transformer = createTransformer(transformers, store, action, true);
                if (transformer === null) {
                    return;
                }
                history.clearFuture();
                history.push(transformer);
            };
        };
    };
}

var index = {
    REDO: REDO,
    UNDO: UNDO,
    createMiddleware: createMiddleware,
    redo: redo,
    undo: undo
};

export default index;
//# sourceMappingURL=index.es.js.map
